﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace QuakeViewer
{
    public partial class Form1 : Form
    {
        List<QuakeData> quakeList = null;

        public Form1()
        {
            InitializeComponent();

            // ListView
            listView1.Columns.AddRange(QuakeData.ListViewColumnHeader());
            listView1.FullRowSelect = true;
            listView1.View = View.Details;
            listView1.Items.Clear();
        }

        private void LoadFile(String filename)
        {
            quakeList = QuakeImporter.LoadCSV(filename);
            if (quakeList == null) return;

            foreach (QuakeData q in quakeList)
            {
                System.Diagnostics.Trace.WriteLine("Deserialize: " + q.ToString());
                listView1.Items.Add(q.ToListViewItem());
            }
        }

        private void WriteFile(String filename)
        {
            if (quakeList == null) return;
            QuakeImporter.WriteCSV("sorted.csv", quakeList);
        }

        private void Add(QuakeData q)
        {
            if (quakeList == null) return;
            this.quakeList.Add(q);
        }

        private void Remove(int index)
        {
            if (quakeList == null) return;
            this.quakeList.RemoveAt(index);
        }

        public void Sort(int index)
        {
            switch (index)
            {
                case 0:
                    System.Console.WriteLine("sort by default");
                    this.quakeList.Sort(); // default - using Comparable
                    break;
                case 1:
                    System.Console.WriteLine("sort by Latitude");
                    this.quakeList.Sort(new QuakeLatComparer());
                    break;
                case 2:
                    System.Console.WriteLine("sort by Longitude");
                    this.quakeList.Sort(new QuakeLonComparer());
                    break;
                case 3:
                    System.Console.WriteLine("sort by Depth");
                    this.quakeList.Sort(new QuakeDepthComparer());
                    break;
                case 4:
                    System.Console.WriteLine("sort by Magnitude");
                    this.quakeList.Sort(new QuakeMagnitudeComparer());
                    break;
                case 5:
                    System.Console.WriteLine("sort by MagType");
                    this.quakeList.Sort((p, q) => p.MagType.CompareTo(q.MagType));
                    break;
                case 6:
                    System.Console.WriteLine("sort by ID");
                    this.quakeList.Sort((p, q) => p.ID.CompareTo(q.ID));
                    break;
                case 7:
                    System.Console.WriteLine("sort by Place");
                    this.quakeList.Sort((p, q) => p.Place.CompareTo(q.Place));
                    break;
                default:
                    System.Console.WriteLine("unsupported sort mode");
                    break;
            }
        }

        private void listView1_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            // set the column number that is to be sorted; default to ascending.
            listView1.ListViewItemSorter = new QuakeListViewItemComparer(e.Column);
            // sort with these new sort options.
            listView1.Sort();

            // sort quakeList as well!!
            Sort(e.Column);
        }

        private void listView1_DoubleClick(object sender, EventArgs e)
        {
            listView1.Items.Remove(listView1.FocusedItem);
            // sort quakeList as well!!
            Remove(listView1.FocusedItem.Index);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            listView1.Items.Clear();
            if (quakeList != null) quakeList.Clear();
            Invalidate();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog1.Filter = "CSV files (*.csv)|*.csv|All files (*.*)|*.*";
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                System.Diagnostics.Trace.WriteLine("Debug: filename=" + openFileDialog1.FileName);
                LoadFile(openFileDialog1.FileName);
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Filter = "CSV files (*.csv)|*.csv|All files (*.*)|*.*";
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                System.Diagnostics.Trace.WriteLine("Debug: filename=" + saveFileDialog1.FileName);
                WriteFile(saveFileDialog1.FileName);
            }
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void findToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // 폼 중복생성 방지
            bool formExist = false;

            foreach (Form f in Application.OpenForms)
            {
                if (f.GetType() == typeof(Form2))
                {
                    f.Activate();
                    formExist = true;
                }
            }

            if (!formExist)
            {
                Form2 f = new Form2(quakeList);
                f.Text = "Quake 찾기";
                f.Owner = this;
                f.Show();
            }
        }
    }
}
