﻿using System;
using System.Collections;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace QuakeViewer
{
    class ListViewItemComparer : IComparer
    {
        public enum SortType : int { Text, Number, DateTime }
        private SortType type = SortType.Text; // default = text

        private int column = 0;

        public ListViewItemComparer()
            : this(0, SortType.Text)
        {
        }

        public ListViewItemComparer(int column, SortType type)
        {
            this.column = column;
            this.type = type;
        }

        public int Compare(object x, object y)
        {
            int result = 0;
            try
            {
                // Cast the objects to be compared to ListViewItem objects        
                ListViewItem listviewX = (ListViewItem)x;
                ListViewItem listviewY = (ListViewItem)y;

                if (type == SortType.Text)
                {
                    result = String.Compare(listviewX.SubItems[column].Text, listviewY.SubItems[column].Text);
                }
                else if (type == SortType.Number)
                {
                    double numX, numY;
                    bool value = Double.TryParse(((ListViewItem)x).SubItems[column].Text, out numX);
                    value = Double.TryParse(((ListViewItem)y).SubItems[column].Text, out numY);
                    result = numX.CompareTo(numY);
                }
                else if (type == SortType.DateTime)
                {
                    DateTime timeX, timeY;
                    bool value = DateTime.TryParse(((ListViewItem)x).SubItems[column].Text, out timeX);
                    value = DateTime.TryParse(((ListViewItem)y).SubItems[column].Text, out timeY);
                    result = timeX.CompareTo(timeY);
                }
            }
            catch
            {
                result = String.Compare(((ListViewItem)x).SubItems[column].Text, ((ListViewItem)y).SubItems[column].Text);
            }
            return result;
        }
    }
}
