﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace QuakeViewer
{
    public class QuakeData : IComparable<QuakeData>, IEquatable<QuakeData>
    {
        public DateTime? Time
        {
            get;
            set;
        }
        public double? Latitude
        {
            get;
            set;
        }
        public double? Longitude
        {
            get;
            set;
        }
        public double? Depth
        {
            get;
            set;
        }
        public double? Magnitude
        {
            get;
            set;
        }

        public string MagType
        {
            get;
            set;
        }

        public int? Nst
        {
            get;
            set;
        }

        public decimal? Gap
        {
            get;
            set;
        }

        public decimal? Dmin
        {
            get;
            set;
        }

        public decimal? Rms
        {
            get;
            set;
        }

        public string Net
        {
            get;
            set;
        }

        public string ID
        {
            get;
            set;
        }

        public DateTime? Updated
        {
            get;
            set;
        }

        public string Place
        {
            get;
            set;
        }

        public string Type
        {
            get;
            set;
        }
        public double? HorizontalError
        {
            get;
            set;
        }

        public double? DepthError
        {
            get;
            set;
        }

        public double? MagError
        {
            get;
            set;
        }

        public int? MagNst
        {
            get;
            set;
        }

        public string Status
        {
            get;
            set;
        }

        public string LocationSource
        {
            get;
            set;
        }
        
        public string MagSource
        {
            get;
            set;
        }

        double r, g, b;

        public double Red
        {
            get
            {
                return r;
            }
        }

        public double Green
        {
            get
            {
                return g;
            }
        }

        public double Blue
        {
            get
            {
                return b;
            }
        }

        public QuakeData() :this(DateTime.Now, 0.0, 0.0, 0.0, 0.0, "", "", "")
        {
        }

        public QuakeData(DateTime? time, double? lat, double? lon, double? depth, double? magnitude, string magType, string id, string place)
        {
            this.Time = time;
            this.Latitude = lat;
            this.Longitude = lon;
            this.Depth = depth;
            this.Magnitude = magnitude;
            this.MagType = magType;
            this.ID = id;
            this.Place = place;
        }

        public QuakeData(DateTime? time, double? lat, double? lon, double? depth, double? magnitude, string magType, int? nst, decimal? gap, decimal? dmin, decimal? rms, string net, string id, DateTime? updated, string place, string type, double? he, double? de, double? me, int? magNst, string status, string locSource, string magSource)
        {
            this.Time = time;
            this.Latitude = lat;
            this.Longitude = lon;
            this.Depth = depth;
            this.Magnitude = magnitude;
            this.MagType = magType;
            this.Nst = nst;
            this.Gap = gap;
            this.Dmin = dmin;
            this.Rms = rms;
            this.Net = net;
            this.ID = id;
            this.Updated = updated;
            this.Place = place;
            this.Type = type;
            this.HorizontalError = he;
            this.DepthError = de;
            this.MagError = me;
            this.MagNst = magNst;
            this.Status = status;
            this.LocationSource = locSource;
            this.MagSource = magSource;
        }

        // compute compressed depth
        public double GetCompressedDepth()         
        {
            double depth = Depth.GetValueOrDefault();
            return depth / 20.0;
        }

        // compute color based on magnitude
        public void ComputeColor()
        {
            double magnitude = Magnitude.GetValueOrDefault();
            if (magnitude <= 3.0) {
                // BLUE
                r = 0.0; g = 0.0; b = 1.0;
            } 
            else if ((magnitude > 3.0) && (magnitude <= 4.25)) {
                // BLUE->CYAN
                r = 0.0; g = (magnitude - 3.0)/1.25; b = 1.0;
            } 
            else if ((magnitude > 4.25) && (magnitude <= 5.5)) {
                // CYAN->GREEN
                r = 0.0; g = 1.0; b = 1 - (magnitude - 4.0)/1.25;
            } 
            else if ((magnitude > 5.5) && (magnitude <= 6.75)) {
                // GREEN->YELLOW
                r = (magnitude - 5.0)/1.25; g = 1.0; b = 0.0;
            }
            else if ((magnitude > 6.75) && (magnitude <= 8.0)) {
                // YELLOW->RED
                r = 1.0; g = 1 - (magnitude - 6.0)/1.25; b = 0.0;
            }
            else if (magnitude > 8.0) {
                // RED
                r = 1.0; g = 0.0; b = 0.0;
            }
        }

        // System.Object.ToString method override
        public override string ToString()
        {
            String nstStr = "";
            String gapStr = "";
            String dminStr = "";
            String rmsStr = "";
            String heStr = "";
            String deStr = "";
            String meStr = "";
            String magNstStr = "";
            if (Nst != null) nstStr = Nst.GetValueOrDefault().ToString();
            if (Gap != null) gapStr = Gap.GetValueOrDefault().ToString();
            if (Dmin != null) dminStr = Dmin.GetValueOrDefault().ToString();
            if (Rms != null) rmsStr = Rms.GetValueOrDefault().ToString();
            if (HorizontalError != null) heStr = HorizontalError.GetValueOrDefault().ToString();
            if (DepthError != null) deStr = DepthError.GetValueOrDefault().ToString();
            if (MagError != null) meStr = MagError.GetValueOrDefault().ToString();
            if (MagNst != null) magNstStr = MagNst.GetValueOrDefault().ToString();

            return string.Format("{0},{1},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11},{12},{13},{14},{15},{16},{17},{18},{19},{20},{21}", 
                Time.GetValueOrDefault(), Latitude.GetValueOrDefault(), Longitude.GetValueOrDefault(), Depth.GetValueOrDefault(), Magnitude.GetValueOrDefault(), 
                MagType, nstStr, gapStr, dminStr, rmsStr, Net, ID, Updated.GetValueOrDefault(), Place, 
                Type, heStr, deStr, meStr, magNstStr, Status, LocationSource, MagSource); // csv file format으로 출력
        }

        // needed for sort
        #region IComparable<QuakeData> QuakeSort
        public int CompareTo(QuakeData other)
        {
            return Time.GetValueOrDefault().CompareTo(other.Time.GetValueOrDefault());
        }
        #endregion

        // ==
        public bool Equals(QuakeData other)
        {
            return Time.Equals(other.Time) && Latitude.Equals(other.Latitude) && Longitude.Equals(other.Longitude) && Depth.Equals(other.Depth) && Magnitude.Equals(other.Magnitude) ;
        }

        // needed for ListView
        #region ListViewItem
        public static ColumnHeader[] ListViewColumnHeader()
        {
            // column headers
            ColumnHeader[] headers = new ColumnHeader[]
            {
                new ColumnHeader() { Text = "DateTime", Width = 150 },
                new ColumnHeader() { Text = "Latitude", Width = 100 },
                new ColumnHeader() { Text = "Longitude", Width = 100 },
                new ColumnHeader() { Text = "Depth", Width = 80 },
                new ColumnHeader() { Text = "Magnitude", Width = 80 },
                new ColumnHeader() { Text = "MagType", Width = 80 },
                new ColumnHeader() { Text = "ID", Width = 80 },
                new ColumnHeader() { Text = "Place", Width = 200 }
            };
            return headers;
        }

        public ListViewItem ToListViewItem()
        {
            ListViewItem item = new ListViewItem(this.Time.ToString());
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.Latitude.ToString()));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.Longitude.ToString()));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.Depth.ToString()));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.Magnitude.ToString()));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.MagType.ToString()));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.ID));
            item.SubItems.Add(new ListViewItem.ListViewSubItem(item, this.Place));
            return item;
        }
        #endregion

        // parse data
        #region Parse
        public delegate bool TryParseHandler<T>(string str, out T value);

        public static T? GetValue<T>(String str, TryParseHandler<T> handler) where T : struct
        {
            if (String.IsNullOrEmpty(str))
                return null;

            T val;
            if (handler(str, out val))
                return val;
            return null;
        }

        public static QuakeData Parse(string[] items)
        {
            QuakeData quake = new QuakeData();
            quake.Time = GetValue<DateTime>(items[0], DateTime.TryParse);
            quake.Latitude = GetValue<Double>(items[1], Double.TryParse);
            quake.Longitude = GetValue<Double>(items[2], Double.TryParse);
            quake.Depth = GetValue<Double>(items[3], Double.TryParse);
            quake.Magnitude = GetValue<Double>(items[4], Double.TryParse);
            quake.MagType = items[5];
            quake.Nst = GetValue<Int32>(items[6], Int32.TryParse);
            quake.Gap = GetValue<Decimal>(items[7], Decimal.TryParse);
            quake.Dmin = GetValue<Decimal>(items[8], Decimal.TryParse);
            quake.Rms = GetValue<Decimal>(items[9], Decimal.TryParse);
            quake.Net = items[10];
            quake.ID = items[11];
            quake.Updated = GetValue<DateTime>(items[12], DateTime.TryParse);

            if (items.Length == 22)
            {
                quake.Place = items[13];
                quake.Type = items[14];
                quake.HorizontalError = GetValue<Double>(items[15], Double.TryParse);
                quake.DepthError = GetValue<Double>(items[16], Double.TryParse);
                quake.MagError = GetValue<Double>(items[17], Double.TryParse);
                quake.MagNst = GetValue<Int32>(items[18], Int32.TryParse);
                quake.Status = items[19];
                quake.LocationSource = items[20];
                quake.MagSource = items[21];
            }
            else if (items.Length == 23)
            {
                quake.Place = items[13] + "," + items[14];
                quake.Type = items[15];
                quake.HorizontalError = GetValue<Double>(items[16], Double.TryParse);
                quake.DepthError = GetValue<Double>(items[17], Double.TryParse);
                quake.MagError = GetValue<Double>(items[18], Double.TryParse);
                quake.MagNst = GetValue<Int32>(items[19], Int32.TryParse);
                quake.Status = items[20];
                quake.LocationSource = items[21];
                quake.MagSource = items[22];
            }

            return quake;
        }
        #endregion
    }
}
