﻿using System;
using System.Collections.Generic;

namespace QuakeViewer
{
    public class QuakeManager
    {
        private List<QuakeData> quakeList = null;

        public QuakeManager()
        {
            quakeList = new List<QuakeData>();
        }

        public QuakeManager(List<QuakeData> quakeList)
        {
            this.quakeList = quakeList;
        }

        public List<QuakeData> ToList()
        {
            return quakeList;
        }

        public void LoadCSV(String filename)
        {
            this.quakeList = QuakeImporter.LoadCSV(filename);
        }

        public void WriteCSV(String filename)
        {
            QuakeImporter.WriteCSV(filename, this.quakeList);
        }

        // add new MetricConverter
        public void AddRange(List<QuakeData> quakeList)
        {
            this.quakeList.AddRange(quakeList);
        }

        // add new MetricConverter
        public void Add(QuakeData converter)
        {
            this.quakeList.Add(converter);
        }

        // print all MetricConverter
        public void Print()
        {
            this.quakeList.ForEach(c => System.Console.WriteLine(c));
        }

        // sort by
        public void Sort(int index)
        {
            switch (index)
            {
                case 0:
                    System.Console.WriteLine("sort by default");
                    this.quakeList.Sort(); // default - using Comparable
                    break;
                case 1:
                    System.Console.WriteLine("sort by Latitude");
                    this.quakeList.Sort(new QuakeLatComparer());
                    break;
                case 2:
                    System.Console.WriteLine("sort by Longitude");
                    this.quakeList.Sort(new QuakeLonComparer());
                    break;
                case 3:
                    System.Console.WriteLine("sort by Depth");
                    this.quakeList.Sort(new QuakeDepthComparer());
                    break;
                case 4:
                    System.Console.WriteLine("sort by Magnitude");
                    this.quakeList.Sort(new QuakeMagnitudeComparer());
                    break;
                case 5:
                    System.Console.WriteLine("sort by MagType");
                    this.quakeList.Sort((p,q) => p.MagType.CompareTo(q.MagType));
                    break;
                case 6:
                    System.Console.WriteLine("sort by ID");
                    this.quakeList.Sort((p, q) => p.ID.CompareTo(q.ID));
                    break;
                case 7:
                    System.Console.WriteLine("sort by Place");
                    this.quakeList.Sort((p, q) => p.Place.CompareTo(q.Place));
                    break;
                default:
                    System.Console.WriteLine("unsupported sort mode");
                    break;
            }
        }

        // remove at
        public void RemoveAt(int index)
        {
            quakeList.RemoveAt(index);
        }

        // removeall if matches
        public void RemoveIf(Predicate<QuakeData> func)
        {
            quakeList.RemoveAll(func);
        }

        public List<QuakeData> findAllIf(Predicate<QuakeData> func)
        {
            return quakeList.FindAll(func);
        }
    }
}
